@@ -1881,9 +1881,12 @@ galera::ReplicatorSMM::update_state_uuid (const wsrep_uuid_t& uuid)
         *(const_cast<wsrep_uuid_t*>(&state_uuid_)) = uuid;
 
         std::ostringstream os; os << state_uuid_;
-
-        strncpy(const_cast<char*>(state_uuid_str_), os.str().c_str(),
-                sizeof(state_uuid_str_));
+        // Copy only non-nil terminated part of the source string
+        // and terminate the string explicitly to silence a warning
+        // generated by Wstringop-truncation
+        char* str(const_cast<char*>(state_uuid_str_));
+        strncpy(str, os.str().c_str(), sizeof(state_uuid_str_) - 1);
+        str[sizeof(state_uuid_str_) - 1] = '\0';
     }
 
     st_.set(uuid, WSREP_SEQNO_UNDEFINED, safe_to_bootstrap_);
