@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2016 Codership Oy <info@codership.com>
+ * Copyright (C) 2011-2017 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -10,7 +10,8 @@
 #include "gcache_bh.hpp"
 #include "gcache_rb_test.hpp"
 
-#include "gu_logger.hpp"
+#include <gu_logger.hpp>
+#include <gu_throw.hpp>
 
 using namespace gcache;
 
@@ -20,7 +21,10 @@ static size_t      const BH_SIZE(sizeof(gcache::BufferHeader));
 
 typedef MemOps::size_type size_type;
 
-static size_type ALLOC_SIZE(size_type s) { return (s + BH_SIZE); }
+static size_type ALLOC_SIZE(size_type s)
+{
+    return MemOps::align_size(s + BH_SIZE);
+}
 
 START_TEST(test1)
 {
@@ -41,7 +45,7 @@ START_TEST(test1)
         fail(os.str().c_str());
     }
 
-    void* buf1 = rb.malloc (ALLOC_SIZE(3));
+    void* buf1 = rb.malloc (MemOps::align_size(rb_size/2 + 1));
     fail_if (NULL != buf1); // > 1/2 size
 
     buf1 = rb.malloc (ALLOC_SIZE(1));
@@ -60,20 +64,33 @@ START_TEST(test1)
     fail_if (BH_is_released(bh2));
 
     void* tmp = rb.realloc (buf1, ALLOC_SIZE(2));
+    // anything <= MemOps::ALIGNMENT should fit into original buffer
+    fail_if(tmp != buf1 && MemOps::ALIGNMENT > 1);
+
+    tmp = rb.realloc (buf1, ALLOC_SIZE(MemOps::ALIGNMENT + 1));
+    // should require new buffer for which there's no space
     fail_if (bh2->seqno_g != SEQNO_NONE);
     fail_if (NULL != tmp);
 
     BH_release(bh2);
     rb.free (bh2);
 
-    tmp = rb.realloc (buf1, ALLOC_SIZE(2));
-    fail_if (NULL != tmp);
+    tmp = rb.realloc (buf1, ALLOC_SIZE(3));
+    if (MemOps::ALIGNMENT > 2)
+    {
+        fail_if (NULL == tmp);
+        fail_if (buf1 != tmp);
+    }
+    else
+    {
+        fail_if (NULL != tmp);
+    }
 
     BH_release(bh1);
     rb.free (bh1);
     fail_if (!BH_is_released(bh1));
 
-    buf1 = rb.malloc (1 + BH_SIZE);
+    buf1 = rb.malloc(ALLOC_SIZE(1));
     fail_if (NULL == buf1);
 
     tmp = rb.realloc (buf1, ALLOC_SIZE(2));
@@ -138,7 +155,8 @@ START_TEST(recovery)
         gu::UUID     gid;
         RingBuffer   rb;
 
-        rb_ctx(size_t s) : size(s), s2p(), gid(), rb(RB_NAME,size,s2p,gid,true){}
+        rb_ctx(size_t s, bool recover = true) :
+            size(s), s2p(), gid(GID), rb(RB_NAME, size, s2p, gid, recover) {}
 
         void seqno_assign (seqno2ptr_t& s2p, void* const ptr,
                            seqno_t const g, seqno_t const d)
@@ -228,7 +246,7 @@ START_TEST(recovery)
         seqno_max = ctx.s2p.rbegin()->first;
     }
 
-    /* What we have now is |111222***444|----| */
+    /* What we have now is |111222444***|----| */
     /* Reopening of the file should:
      * 1) discard messages 1, 2 since there is a hole at 3. Only 4 should remain
      * 2) trim the trailing unordered message
@@ -394,9 +412,99 @@ START_TEST(recovery)
         fail_if(ctx.s2p.begin()->first  != seqno_max);
         fail_if(ctx.s2p.rbegin()->first != seqno_max);
 
+        fail_if(seqno_max < 1);
+        fail_if(seqno_min != seqno_max);
+    }
+
+    ::unlink(RB_NAME.c_str());
+
+    /* test for singe segment in the middle */
+    void* third_buffer(NULL);
+    {
+        rb_ctx ctx(rb_3size, false);
+
+        fail_if (ctx.rb.size() != ctx.size,
+                 "Expected %zd, got %zd", ctx.size, ctx.rb.size());
+
+        if (ctx.gid != GID)
+        {
+            std::ostringstream os;
+            os << "Expected GID: " << GID << ", got: " << ctx.gid;
+            fail(os.str().c_str());
+        }
+
+        fail_if(!ctx.s2p.empty());
+
+        void* m(ctx.add_msg(msgs[3]));
+        fail_if (NULL == m);
+        fail_if (ctx.s2p.find(msgs[3].g) != ctx.s2p.end());
+
+        m = ctx.add_msg(msgs[4]);
+        fail_if (NULL == m);
+        fail_if (ctx.s2p.find(msgs[4].g)->second != m);
+
+        m = ctx.add_msg(msgs[5]);
+        fail_if (NULL == m);
+        fail_if (ctx.s2p.find(msgs[5].g) != ctx.s2p.end());
+        third_buffer = m;
+
+        fail_if(ctx.s2p.empty());
+        fail_if(ctx.s2p.size() != 1);
         seqno_min = ctx.s2p.begin()->first;
         seqno_max = ctx.s2p.rbegin()->first;
+        fail_if(seqno_min != seqno_max);
     }
+
+    /* now the situation should be |***444***| - only one segment, in the middle,
+     * reopen the file with a known position */
+    {
+        rb_ctx ctx(rb_3size);
+
+        fail_if (ctx.rb.size() != ctx.size,
+                 "Expected %zd, got %zd", ctx.size, ctx.rb.size());
+
+        if (ctx.gid != GID)
+        {
+            std::ostringstream os;
+            os << "Expected GID: " << GID << ", got: " << ctx.gid;
+            fail(os.str().c_str());
+        }
+
+        fail_if(ctx.s2p.empty());
+        fail_if(ctx.s2p.size() != 1);
+        fail_if(seqno_min != ctx.s2p.begin()->first);
+        fail_if(seqno_max != ctx.s2p.rbegin()->first);
+        fail_if(seqno_min != seqno_max);
+    }
+
+    /* now the situation should be |---444---| - only one segment, in the middle,
+     * reopen the file a second time - to trigger a rollover bug */
+    {
+        rb_ctx ctx(rb_3size);
+
+        fail_if (ctx.rb.size() != ctx.size,
+                 "Expected %zd, got %zd", ctx.size, ctx.rb.size());
+
+        if (ctx.gid != GID)
+        {
+            std::ostringstream os;
+            os << "Expected GID: " << GID << ", got: " << ctx.gid;
+            fail(os.str().c_str());
+        }
+
+        fail_if(ctx.s2p.empty());
+        fail_if(ctx.s2p.size() != 1);
+        fail_if(seqno_min != ctx.s2p.begin()->first);
+        fail_if(seqno_max != ctx.s2p.rbegin()->first);
+        fail_if(seqno_min != seqno_max);
+
+        // must be allocated right after the recovered buffer
+        void* m(ctx.add_msg(msgs[3]));
+        fail_if (NULL == m);
+        fail_if (third_buffer != m);
+    }
+
+    ::unlink(RB_NAME.c_str());
 }
 END_TEST
 
