@@ -474,8 +474,8 @@ gcs_group_handle_comp_msg (gcs_group_t* group, const gcs_comp_msg_t* comp)
                 /* initialize node ID to the one given by the backend - this way
                  * we'll be recognized as coming from prev. conf. in node array
                  * remap below */
-                strncpy ((char*)group->nodes[0].id, new_nodes[0].id,
-                         sizeof (new_nodes[0].id) - 1);
+                strncpy (group->nodes[0].id, new_nodes[0].id,
+                         sizeof (group->nodes[0].id));
                 group->nodes[0].segment = new_nodes[0].segment;
             }
         }
@@ -542,8 +542,9 @@ gcs_group_handle_uuid_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
 
     if (GCS_GROUP_WAIT_STATE_UUID == group->state &&
         0 == msg->sender_idx /* check that it is from the representative */) {
-        group->state_uuid = *(gu_uuid_t*)msg->buf;
-        group->state      = GCS_GROUP_WAIT_STATE_MSG;
+//        group->state_uuid = *(gu_uuid_t*)msg->buf;
+        gu_uuid_copy(&group->state_uuid, (const gu_uuid_t*)msg->buf);
+        group->state = GCS_GROUP_WAIT_STATE_MSG;
     }
     else {
         gu_warn ("Stray state UUID msg: " GU_UUID_FORMAT
