@@ -1,10 +1,11 @@
 /*
- * Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2017 Codership Oy <info@codership.com>
  *
  * $Id$
  */
 
 #include "gcs_params.hpp"
+#include "gcs_fc.hpp" // gcs_fc_hard_limit_fix
 
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
@@ -19,6 +20,9 @@ const char* const GCS_PARAMS_MAX_PKT_SIZE      = "gcs.max_packet_size";
 const char* const GCS_PARAMS_RECV_Q_HARD_LIMIT = "gcs.recv_q_hard_limit";
 const char* const GCS_PARAMS_RECV_Q_SOFT_LIMIT = "gcs.recv_q_soft_limit";
 const char* const GCS_PARAMS_MAX_THROTTLE      = "gcs.max_throttle";
+#ifdef GCS_SM_DEBUG
+const char* const GCS_PARAMS_SM_DUMP           = "gcs.sm_dump";
+#endif /* GCS_SM_DEBUG */
 
 static const char* const GCS_PARAMS_FC_FACTOR_DEFAULT         = "1.0";
 static const char* const GCS_PARAMS_FC_LIMIT_DEFAULT          = "16";
@@ -57,7 +61,9 @@ gcs_params_register(gu_config_t* conf)
                           GCS_PARAMS_RECV_Q_SOFT_LIMIT_DEFAULT);
     ret |= gu_config_add (conf, GCS_PARAMS_MAX_THROTTLE,
                           GCS_PARAMS_MAX_THROTTLE_DEFAULT);
-
+#ifdef GCS_SM_DEBUG
+    ret |= gu_config_add (conf, GCS_PARAMS_SM_DUMP, "0");
+#endif /* GCS_SM_DEBUG */
     return ret;
 }
 
@@ -200,7 +206,8 @@ gcs_params_init (struct gcs_params* params, gu_config_t* config)
     int64_t tmp;
     if ((ret = params_init_int64 (config, GCS_PARAMS_RECV_Q_HARD_LIMIT, 0, 0,
                                   &tmp))) return ret;
-    params->recv_q_hard_limit = tmp * 0.9; // allow for some meta overhead
+    params->recv_q_hard_limit = tmp * gcs_fc_hard_limit_fix;
+    // allow for some meta overhead
 
     if ((ret = params_init_bool (config, GCS_PARAMS_FC_MASTER_SLAVE,
                                  &params->fc_master_slave))) return ret;
